/********************						***********************/
//
//	Player PRO 4.4x -- MAD Music Driver Definition
//
//	Library Version 3.0
//
//	To use with MusicLibrary for Think C & CodeWarrior
//
//	Antoine ROSSET
//	16 Tranchees
//	1206 GENEVA
//	SWITZERLAND
//	
//	FAX:			(+41 22) 346 11 97
//	Compuserve:		100277,164
//	Internet: 		rosset@dial.eunet.ch
//
/********************						***********************/

#ifndef __RDRIVERH__
#define __RDRIVERH__

#ifndef __MADH__
#include "MAD.h"
#endif

#ifndef __SOUND__
#include <Sound.h>
#endif

#ifndef __RETRACE__
#include <retrace.h>
#endif

/********************						***********************/
/*** 			   		 Define List							***/
/********************						***********************/

#define MAX_ARP 			3
#define NUMBER_FINETUNES 	16
#define NUMBER_NOTES		84
#define AMIGA_CLOCKFREQ2	3575872L

#define MIN_PITCH			13L
#define MAX_PITCH			1918L

#define MIN_VOLUME			0
#define MAX_VOLUME			64
#define MAXTRACK			32
#define MAXINSTRU			64
#define MAXPATTERN			200
#define MAXPATTERNSIZE		900

#define POSPITCH		170
#define	SoundActive  	0x27E
#define ASCBase 		0x0CC0

/********************						***********************/
/*** 			    Error send by MAD Functions					***/
/********************						***********************/

enum
{
	MADNeedMemory 				= -1,
	MADReadingErr				= -2,
	MADIncompatibleFile			= -3,
	MADLibraryNotInitialized	= -4,
	MADNeedBGGB					= -5,
	MADParametersErr			= -6,
	MADUnknowErr				= -7,
	MADSoundManagerErr			= -8
};

/********************						***********************/
/*** 		   Interruption VBL for ASC/AWAC Drivers			***/
/********************						***********************/

#if defined(powerc) || defined (__powerc)
#pragma options align=mac68k
#endif
struct VBLRec {
    VBLTask     theVBLTask;
    long        VBLA5;
};
#if defined(powerc) || defined(__powerc)
#pragma options align=reset
#endif
typedef struct VBLRec VBLRec, *VBLRecPtr;

/********************						***********************/
/*** 			     Voice structure definition					***/
/********************						***********************/

#if defined(powerc) || defined (__powerc)
#pragma options align=mac68k
#endif
struct VoiceActive
{
		Ptr			MaxPtr;
		Ptr			InstruVoice;
		Ptr			StartPtr;
		long		loopWord;
		long		loopWords;
		short		Instru;
		short		InstruOld;
		short		fineTune;
		short		Amiga;
		short		AmigaOld;
		short		pitch;
		short 		realpitch;
		short		Prerealpitch;
		short		volume;
		short 		effect;
		Byte		arg;
		short 		arp[MAX_ARP];
		short 		arpindex;
		short		viboffset;
		short 		vibdepth;
		short 		slide;
		short 		pitchgoal;
		short 		pitchrate;
		short		oldpitchrate;
		short 		volumerate;
		short 		vibrate;
		short		oldvibrate;
		short 		retrig;
		short		ID;
		short		amplitude;
		Ptr			samplePtr;
		Boolean		repeat;
};
#if defined(powerc) || defined(__powerc)
#pragma options align=reset
#endif
typedef		struct VoiceActive	VoiceActive;

/********************						***********************/
/*** 		Music description - used in Import/Export filter	***/
/********************						***********************/

#if defined(powerc) || defined (__powerc)
#pragma options align=mac68k
#endif
struct	MADPartition
{
	MADSpec					*header;
	struct	MusicPattern	*partition[ MAXPATTERN];			// Patterns
	Ptr						instrument[ MAXINSTRU];				// Instruments
};
#if defined(powerc) || defined(__powerc)
#pragma options align=reset
#endif
typedef		struct MADPartition	MADPartition;

/********************						***********************/
/*** 			     Driver Settings definition					***/
/********************						***********************/

enum
{
	ASCSoundDriver = 1,
	AWACSoundDriver,
	MIDISoundDriver,
	SoundManagerDriver	
};

enum
{
	MonoOutPut = 1,
	StereoOutPut,
	DeluxeStereoOutPut,
	PolyPhonic					// Do NOT use it !
};

#if defined(powerc) || defined (__powerc)
#pragma options align=mac68k
#endif
struct MADDriverSettings
{
	short					Tracks;								// Active tracks from 2 to 32, automatically setup when a new music is loaded
	short					OutPutBits;							// 8 or 16 Bits
	unsigned long			OutPutRate;							// Fixed number, by example : rate44khz, rate22050hz, rate22khz, rate11khz, rate11025hz
	short					OutPutMode;							// MonoOutPut, StereoOutPut or DeluxeStereoOutPut ?
	short					DriverMode;							// ASCSoundDriver, AWACSoundDriver, MIDISoundDriver or SoundManagerDriver
	Boolean					AntiAliasing;						// Use AntiAliasing filter ?
	Boolean					RepeatMusic;						// If music finished, repeat it or stop.
	Boolean					PtrSystem;							// Allocate memory in Application Heap (false) or System Heap (true)
};
#if defined(powerc) || defined(__powerc)
#pragma options align=reset
#endif
typedef struct MADDriverSettings MADDriverSettings;

/********************						***********************/
/*** 		Global structure : PlayerPRO variables				***/
/********************						***********************/

#if defined(powerc) || defined (__powerc)
#pragma options align=mac68k
#endif
struct MADDriverRec
{
	long					IDType;								// IDType = 'MADD' -- READ ONLY --

	/**********************/
	/** Public variables **/
	/**********************/

	MADDriverSettings		DriverSettings;						// Driver SetUp -- READ ONLY --

	/**  Current music in memory, loaded with RLoadMusic() by example **/

	MADSpec					*header;							// MAD Header -- READ ONLY --
	struct	MusicPattern	*partition[ MAXPATTERN];			// Patterns -- READ ONLY --
	Ptr						instrument[ MAXINSTRU];				// Instruments -- READ ONLY --

	/**  Drivers variables **/

	VoiceActive				theVoiceActive[ MAXTRACK];			// Current driver voices -- READ ONLY --
	Boolean					MusiqueFertig;						// Is your music finished? -- READ ONLY --
	short					Tube[ MAXTRACK];					// Used in 'Tracks View' Window - View menu 
	short					PartitionReader;					// Current position in pattern (0...64)
	short					Pat;								// Current ID Pattern, see 'Patterns list'
	short					PL;									// Current position in partition, see 'Partition list'
	short					VolExt[ MAXTRACK];					// Volumes settings, see 'Adaptators' window
	short					speed;								// Current speed, see speed Effect
	short					finespeed;							// Current finespeed, see speed Effect
	short					InstruTube[ MAXINSTRU];				// Used in 'Instrument View' Window - View menu
	short					VExt;								// External music speed, see 'Adaptators' window
	short					FreqExt;							// External music pitch, see 'Adaptators' window
	Boolean					Reading;							// Reading indicator
	short					LeftRight[ MAXTRACK];				// Left/Right % for Deluxe Driver

	/** Private variables **/
	/* DO NOT MODIFY OR USE these variables */

	short					smallcounter, trackDiv;
	long					FREQBASE;
	short					**FreqHandle, InstruActif[ MAXINSTRU];
	Ptr						SysHeapPtr, Vol, IntDataPtr, OscilloWavePtr;
	Boolean					JumpToNextPattern;
	long					ASCBUFFER;
	long					BufSize;
	short					pitchTable[ NUMBER_NOTES + 4][ NUMBER_FINETUNES];
	long					VSYNC, BufCounter, BytesToGenerate;
	short					vibrato_table[ 64];
	VBLRec					VBL;
	short					MIDIPortRefNum;
	SndChannelPtr 			pseudoChanAWAC;
	short					InstuNoOld[ MAXTRACK];
	short					NoteOld[ MAXTRACK];
	short					VelocityOld[ MAXTRACK];
	SndDoubleBufferHeader 	TheHeader;
	SndChannelPtr 			MusicChannelPP;
	Ptr						OverShoot;
	long					*DASCBuffer;
	short					*DASCBuffer8;
};
#if defined(powerc) || defined(__powerc)
#pragma options align=reset
#endif
typedef struct MADDriverRec MADDriverRec;

#ifdef MainSystemPlayerPRO
		MADDriverRec		*MADDriver;
#else
extern	MADDriverRec		*MADDriver;
#endif

/********************						***********************/
/*** 					   EFFECTS ID							***/
/********************						***********************/

enum {
		arpeggioE 		= 0,
		downslideE 		= 1,
		upslideE 		= 2,
		portamentoE 	= 3,
		vibratoE 		= 4,
		portaslideE 	= 5,
		vibratoslideE	= 6,
		nothingE 		= 7,
		offsetE 		= 9,
		slidevolE 		= 10,
		fastskipE 		= 11,
		volumeE 		= 12,
		skipE 			= 13,
		extendedE 		= 14,
		speedE 			= 15
	};

/********************						***********************/
/*** 					   FUNCTIONS							***/
/********************						***********************/


#ifdef __cplusplus
extern "C" {
#endif

OSErr	MADInitLibrary( void);										// Library initialisation, you have to CALL this function if you want to use other functions & variables
OSErr	MADDisposeLibrary( void);									// Close Library, close music, close driver, free all memory

OSErr	MADCreateDriver( MADDriverSettings	*DriverInitParam);		// Music Driver initialization and memory allocation
OSErr	MADDisposeDriver();											// Dispose the music driver, use it after RInitMusic()

OSErr	MADPlay( void);												// Play the current music
OSErr	MADStop( void);												// Stop the current music
OSErr	MADReset( void);											// Reset the current music at the start position

OSErr	MADLoadMusicFile( Str255 fName);							// Load a MAD File in the current directory
OSErr	MADLoadMusicRsrc( OSType IDName, short IDNo);				// Load a MAD Rsrc into memory
OSErr	MADLoadMusicPtr( Ptr myPtr);								// Load a MAD Ptr into memory, you can DisposPtr your Ptr after this call
OSErr	MADLoadMusicPartition( MADPartition aPartition);			// Load a MAD partition into memory
OSErr	MADDisposeMusic(void);										// Dispose the current music, use it after RLoadMusic(), RLoadMusicRsrc(), RInstallMADF()


OSErr	ConvertMod2Mad( Ptr aMOD, long Size, MADPartition *theMAD);	// Convert MOD to MAD
Ptr		ConvertMad2Mod( MADPartition *theMAD);						// Convert MAD to MOD
void	ChangeTracks( short);										// Change current tracks number of the music driver
void	CleanDriver();												// Clean the driver
struct Command* GetCommand( short PosX, short	TrackIdX, struct MusicPattern*	tempMusicPat);		// Extract a Command from a MusicPattern	

// Play a sound with an amplitude on a track at a period, with loopStart and loopEnd
// If period == 0, play it at normal frequence 22Khz ( F#5 )
// Period: from 0 to NUMBER_NOTES

OSErr	MADPlaySound( Ptr whichSound, long SoundSize, long whichTrack, long Period, long Amplitude, long loopStart, long loopLength);


/********************						***********************/
/*** 					INTERNAL ROUTINES						***/
/***			    DO NOT use these routines					***/
/********************						***********************/

/*** Deluxe ***/

void 	CreateOverShoot( short ACdriver);
void 	KillOverShoot( short ACdriver);
void 	Sampler16AddDeluxe( VoiceActive *curVoice, register long	*ASCBuffer, MADDriverRec *intDriver);
void 	Sampler16Addin16Deluxe( VoiceActive *curVoice, register long	*ASCBuffer, MADDriverRec *intDriver);
void 	Sample16BufferAddDeluxe( VoiceActive *curVoice, register long	*ASCBuffer, MADDriverRec *intDriver);
void 	Play16StereoDeluxe( MADDriverRec *intDriver);
void 	Sampler8in8AddDeluxe( VoiceActive *curVoice, register short	*ASCBuffer, MADDriverRec *intDriver);
void 	Sampler8in16AddDeluxe( VoiceActive *curVoice, register short	*ASCBuffer, MADDriverRec *intDriver);
void 	Sample8BufferAddDeluxe( VoiceActive *curVoice, register short *ASCBuffer, MADDriverRec *intDriver);
void 	Play8StereoDeluxe( MADDriverRec *intDriver);

/*** 8 Bits ***/

void 	Sampler8in8Add( VoiceActive *curVoice, register Ptr	ASCBuffer, MADDriverRec *intDriver);
void 	Sampler8in16Add( VoiceActive *curVoice, register Ptr	ASCBuffer, MADDriverRec *intDriver);
void 	Sampler8in8AddPoly( VoiceActive *curVoice, register Ptr	ASCBuffer, short chanNo, MADDriverRec *intDriver);
void 	Sampler8in16AddPoly( VoiceActive *curVoice, register Ptr	ASCBuffer, short chanNo, MADDriverRec *intDriver);
void 	Sample8BufferAdd( VoiceActive *curVoice, register Ptr ASCBuffer, MADDriverRec *intDriver);
void 	Sample8BufferAddPoly( VoiceActive *curVoice, register Ptr ASCBuffer, short chanNo, MADDriverRec *intDriver);
void	Play8Mono( MADDriverRec *intDriver);
void 	Play8Stereo( MADDriverRec *intDriver);
void 	Play8PolyPhonic( MADDriverRec *intDriver);

/*** 16 Bits ***/

void 	Sampler16Add( VoiceActive *curVoice, register short	*ASCBuffer, MADDriverRec* intDriver);
void 	Sampler16Addin16( VoiceActive *curVoice, register short	*ASCBuffer, MADDriverRec* intDriver);
void 	Sample16BufferAdd( VoiceActive *curVoice, register short	*ASCBuffer, MADDriverRec* intDriver);
void 	Play16Stereo( MADDriverRec* intDriver);
void 	Sampler16AddPoly( VoiceActive *curVoice, register short	*ASCBuffer, short chanNo, MADDriverRec* intDriver);
void 	Sampler16Addin16Poly( VoiceActive *curVoice, register short	*ASCBuffer, short chanNo, MADDriverRec* intDriver);
void 	Sample16BufferAddPoly( VoiceActive *curVoice, register short	*ASCBuffer, short chanNo, MADDriverRec* intDriver);
void 	Play16PolyPhonic( MADDriverRec* intDriver);
void 	Play16Mono( MADDriverRec* intDriver);

/*** Effects ***/

void 	checkpitch( VoiceActive *ch, Boolean UpDown, MADDriverRec *intDriver);
void 	DoEffect( VoiceActive *ch, short call, MADDriverRec *intDriver);
void 	SetUpEffect( VoiceActive *ch, MADDriverRec *intDriver);

/*** Interruption Functions ***/

void 	NoteAnalyse( MADDriverRec *intDriver);
pascal 	void DMAPlay ( void);
void 	ReadNote( VoiceActive *curVoice, struct Command		*theCommand, MADDriverRec *intDriver);
void 	Filter8Bit( register Byte *myPtr, MADDriverRec *intDriver);
void 	Filter8BitX( register Byte *myPtr, MADDriverRec *intDriver);
void 	Filter16BitX( register short *myPtr, MADDriverRec *intDriver);
pascal 	void MyDoubleBackProc(SndChannelPtr chan, SndDoubleBufferPtr doubleBuffer);
void 	BufferCopyM( MADDriverRec *intDriver);
void 	BufferCopyS( MADDriverRec *intDriver);
void 	NoteOff(short oldIns, short oldN, short oldV, MADDriverRec *intDriver);
void 	SampleMIDI( VoiceActive *curVoice, short channel, short curN, MADDriverRec *intDriver);
void 	CleanDriver( MADDriverRec *intDriver);

/*** General Functions ***/

void	UpdateTracksNumber();
OSErr	CreateVolumeTable( long TrackNo, long OutBit, short OutPutMode);
void	ClearVolumeTable();
void	CreateOverShoot( short OutPutBits);
void	KillOverShoot( short OutPutBits);

OSErr	SetUpFrequence( long, long);
OSErr	ReadInstrument( short, short, Handle, Ptr);
OSErr	DBSndClose();
void	StopChannel();
void	PlayChannel();
void	InstallMODVBL(void);
void	RemoveMODVBL(void);
void	MODRelance(void);
void	Play(void);
void	VIAOn(void);
void 	SndOff(void);
void 	SndOn(void);
void	VIAOn2(void);
void 	VIAOn3(void);
OSErr	InitDBSoundManager( long);
Boolean DirectSave( Ptr myPtr, MADDriverSettings *driverType);
void	ChangeSpeed( void);
void 	ClearFrequence();
void	InstallPersistentVBL( VBLTask *theVBLRec);
void	InstallDMAVBL();
void	RemoveDMAVBL();
OSErr	create_notes_table();
void 	ActiveDMA();
struct 	MusicPattern* DecompressPartitionMAD1( struct MusicPattern* myPat);
struct 	MusicPattern* CompressPartitionMAD1( struct MusicPattern* myPat);
Ptr 	DoMAC3Expand( Ptr sound, long *InOutSize);
Ptr 	DoMAC3Compress( Ptr sound, long *InOutSize);
void 	DMAIntInstall( unsigned long refCon, Ptr handlerAddr, unsigned long vectorBit);
void 	DMAIntRemove( unsigned long vectorBit);
void 	GenerateSound( MADDriverRec *intDriver);
SndChannelPtr CreateSndChannel( long init);

#ifdef __cplusplus
}
#endif
#else
extern	MADDriverRec		*MADDriver;
#endif
