/********************************************************/
/*
	Player PRO 4.4 -- Music Driver EXAMPLE

	Library Version 3.5

	To use with MusicLibrary 3.5 for Think C & CodeWarrior

	Antoine ROSSET
	16 Tranchees
	1206 GENEVA
	SWITZERLAND
	
	FAX: (41 22) 346 11 97
	Compuserve: 100277,164
	Internet: rosset@dial.eunet.ch
*/
/********************************************************/

#include "MAD.h"				// Mad File Definition
#include "RDriver.h"			// Mad Driver functions & globals
#include "gestaltequ.h"			// For GetBestDriver function
#include "sound.h"				// To hear something....

void GetBestDriver( MADDriverSettings	*Init)
{
long			gestaltAnswer;
short			myBit;
NumVersion		nVers;
Boolean			Stereo, StereoMixing, NewSoundManager, hasASC, Audio16;

/***************					****************/
/****** HARDWARE IDENTIFICATION AND CHECK **********/
/***************					****************/

/****************/
/** ASC CHIP ? **/
/****************/

	Gestalt( gestaltHardwareAttr, &gestaltAnswer);
	myBit = gestaltHasASC;
	if( BitTst( &gestaltAnswer, 31-myBit) == false) hasASC = false;
	else hasASC = true;

/**************/
/** STEREO ? **/
/**************/

	Gestalt( gestaltSoundAttr, &gestaltAnswer);
	myBit = gestaltStereoCapability;
	Stereo = BitTst( &gestaltAnswer, 31-myBit);

/******************************/
/** INTERNAL STEREO MIXING ? **/
/******************************/

	myBit = gestaltStereoMixing;
	StereoMixing = BitTst( &gestaltAnswer, 31-myBit);

/***********************/
/** 16 BITS SUPPORT ? **/
/***********************/

	myBit = gestalt16BitSoundIO;
	Audio16 = BitTst( &gestaltAnswer, 31-myBit);

/****************************/
/** SOUND MANAGER >= 3.0 ? **/
/****************************/

	nVers = SndSoundManagerVersion();
	if( nVers.majorRev >= 3) NewSoundManager = true;
	else NewSoundManager = false;
	
	Init->Tracks		= 4;
	Init->AntiAliasing	= false;
	Init->RepeatMusic	= true;
	Init->PtrSystem		= false;
	
	if( Stereo == true && StereoMixing == true)
	{
		Init->OutPutMode = DeluxeStereoOutPut;
		
		if( NewSoundManager == true)
		{
			Init->DriverMode = SoundManagerDriver;
			
			if( Audio16)
			{
				Init->OutPutBits = 16;
				Init->OutPutRate = rate44khz;
			}
			else
			{
				Init->OutPutBits = 8;
				Init->OutPutRate = rate22khz;
			}
		}
		else if( hasASC == true)
		{
			Init->OutPutBits = 8;
			Init->OutPutRate = rate22khz;
			Init->DriverMode = ASCSoundDriver;
		}
	}
	else
	{	
		Init->OutPutMode = MonoOutPut;
	
		if( NewSoundManager == true)
		{
			Init->DriverMode = SoundManagerDriver;
			if( Audio16)
			{
				Init->OutPutBits = 16;
				Init->OutPutRate = rate44khz;
			}
			else
			{
				Init->OutPutBits = 8;
				Init->OutPutRate = rate22khz;
			}
		}
		else if( hasASC == true)
		{
			Init->OutPutBits = 8;
			Init->OutPutRate = rate22khz;
			Init->DriverMode = ASCSoundDriver;
		}
	}
}

/*****************************/
/****** MAIN FUNCTION ********/
/*****************************/

void main( void)
{
long			gestaltAnswer, sndSize;
short			myBit, iFileRefI;
NumVersion		nVers;
Point			where = { -1, -1};
SFReply			reply;
SFTypeList		aType;
OSErr			iErr = noErr;
Ptr				tempPtr;
MADPartition	aPartition;
Boolean			End;

/***************					****************/
/****** 	 	Toolbox Initialization	  **********/
/***************					****************/

InitGraf( &qd.thePort);
InitFonts();
FlushEvents(everyEvent,0);
InitWindows();
TEInit();
InitMenus();
InitCursor();
MaxApplZone();

/*******************************************************************************************/
/****** MAD Library Initialisation : choose the best driver for the current hardware ******/
/******																				  ******/
/****** Standard initialization with 4 tracks...									  ******/
/*******************************************************************************************/

{
MADDriverSettings	init;

init.Tracks				= 4;
init.OutPutBits 		= 8;
init.OutPutRate			= rate22khz;
init.OutPutMode			= StereoOutPut;
init.DriverMode			= SoundManagerDriver;
init.AntiAliasing		= false;
init.RepeatMusic		= false;

MADInitLibrary();
GetBestDriver( &init);
if( MADCreateDriver( &init) != noErr) DebugStr("\pSmall Problem...");
}
/*********************************/
/*********************************/
/*********************************/

/****** Choose a MAD ('MADF') or a MODs ('STrk') File ********/
End = false;

while( End == false)
{
	FlushEvents( everyEvent, 0);
	aType[ 0] = 'MADF';
	aType[ 1] = 'STrk';
	SFGetFile( where, "\p", 0L, 2, aType, 0L, &reply);
	
	if( !reply.good) End = true;
	else
	{
		SetVol( 0L, reply.vRefNum);
		
		/****************/
		/** MOD FILE ? **/
		/****************/

		if( reply.fType == 'STrk')
		{
			iErr = FSOpen( reply.fName, 0, &iFileRefI);
			if( iErr) DebugStr( "\pProblemo ??");
			
			GetEOF( iFileRefI, &sndSize);
			
			tempPtr = NewPtr( sndSize);
			FSRead( iFileRefI, &sndSize, tempPtr);
			FSClose( iFileRefI);
				
			ConvertMod2Mad( tempPtr, sndSize, &aPartition);
			MADLoadMusicPartition( aPartition);
			
			DisposPtr( tempPtr);	tempPtr = 0L;
		}

		/****************/
		/** MAD FILE ? **/
		/****************/

		else if( reply.fType == 'MADF')
		{
			/*** To load a MAD File *********/
			if( MADLoadMusicFile( reply.fName) != noErr) Debugger();
			
			/*
				// *** TO OPEN A MAD RESOURCE ***
				
				// To create a MAD Resource, use Auto-Exec export from Player PRO !!!!
				// And load the resource with: by example...
			
				MADLoadMusicRsrc( 'MADF', 3124); for example...
			*/
		}
		else DebugStr("\pUh Uh... Unknow format");
		
		MADPlay();						// Turn interrupt driver function ON
		MADDriver->Reading = true;		// Read the current partition in memory
		
		while( !Button())
		{
			/** Do what you want here....	**/
			/** Bla bla...					**/
		}
		MADDriver->Reading = false;		// Stop reading current partition
		MADStop();						// Stop driver interrupt function
		
		MADDisposeMusic();				// Dispose the current music
	}
}
MADDisposeDriver();						// Dispose music driver
MADDisposeLibrary();					// Close music library

FlushEvents( everyEvent, 0);
}