/********************************************************/
/*
	Player PRO 4.4 -- Music Driver EXAMPLE

	Library Version 3.5

	To use with MusicLibrary 3.5 for Think C & CodeWarrior

	Antoine ROSSET
	16 Tranchees
	1206 GENEVA
	SWITZERLAND
	
	FAX: (41 22) 346 11 97
	Compuserve: 100277,164
	Internet: rosset@dial.eunet.ch
*/
/********************************************************/

#include "MAD.h"
#include "RDriver.h"
#include "Sound.h"
#include "AIFF.h"

Handle DoExp1to3( Handle sound, unsigned long numSampleFrames)
{
long	i;
Ptr		inState, outState;
Handle	outBuffer;

	outState = NewPtrClear( 128);
	inState = NewPtrClear( 128);
	if( inState == nil)
	{
		/* Error */
	}
	
	outBuffer = NewHandle( numSampleFrames*6);
	if( outBuffer == nil)
	{
		/* Error */
	}
	
	HLock( sound);
	HLock( outBuffer);
	
	Exp1to3( *sound, *outBuffer, numSampleFrames, (void*) inState, (void*) outState, 1, 1);
	
	HUnlock( sound);
	HUnlock( outBuffer);
		
	DisposHandle( sound);
	sound = outBuffer;
	
	DisposPtr( inState);
	DisposPtr( outState);

	return sound;
}

Handle DoExp1to6( Handle sound, unsigned long numSampleFrames)
{
long	i;
Ptr		inState, outState;
Handle	outBuffer;

	outState = NewPtrClear( 128);
	inState = NewPtrClear( 128);
	if( inState == nil)
	{
		/* Error */
	}
	
	outBuffer = NewHandle( numSampleFrames * 6);
	if( outBuffer == nil)
	{
		/* Error */
	}
	
	HLock( sound);
	HLock( outBuffer);
	
	Exp1to6( *sound, *outBuffer, numSampleFrames, (void*) inState, (void*) outState, 1, 1);
	
	HUnlock( sound);
	HUnlock( outBuffer);
		
	DisposHandle( sound);
	sound = outBuffer;
	
	DisposPtr( inState);
	DisposPtr( outState);

	return sound;
}

void ConvertInstrument( register	Byte	*tempPtr,	register long sSize)
{
	register	Byte			val = 0x80;

	while( sSize > 0)
	{
		sSize--;
		*(tempPtr + sSize) += val;
	}
}

/*************************************************************/
/*                       SndToHandle                         */
/* This function is only useful if you use OSErr PlaySound() */
/*************************************************************/
/* This function extract a raw handle from a 'snd ' handle   */
/*************************************************************/
Handle SndToHandle( Handle sound, short	*sampleSize)
{
	Ptr 			soundPtr;
	short 			soundFormat;
	short 			numSynths, numCmds, CompressID;
	long 			offset, MusSize;
	SoundHeaderPtr 	header;
	CmpSoundHeader	*CmpHeader;
	ExtSoundHeader	*ExtHeader;
	SndCommand 		cmd;
	OSErr 			result;
	long			i,x, numFrames;
	Boolean			change = false;
	
	*sampleSize = 8;
	
	HLock( sound);
	soundPtr = *sound;
	
	soundFormat = *(short*)soundPtr;
	
	switch(soundFormat)
	{
		case 1:
			numSynths = ((short*)soundPtr)[1];
			numCmds = *(short*)(soundPtr+4+numSynths*6);
		break;
		
		case 2:
			numSynths = 0;
			numCmds = ((short*)soundPtr)[2];
		break;
		
		default:
			DebugStr("\p SndToHandle");
		break;
	} 

	offset = 6 + 6*numSynths + 8*numCmds;
	header = (SoundHeaderPtr) (StripAddress(*sound) + offset);
	
	switch( header->encode)
	{
		case cmpSH:
			CmpHeader = (CmpSoundHeader*) header;
			CompressID = CmpHeader->compressionID;
			
			*sampleSize = CmpHeader->sampleSize;
			
			MusSize = (*CmpHeader).numFrames;
			
			HLock( sound);
			BlockMove( (*CmpHeader).sampleArea, *sound, MusSize);
			HUnlock( sound);
			
			switch( CompressID )
			{
				case threeToOne:
					MusSize *= 2;
					sound = DoExp1to3( sound, MusSize);
					MusSize *= 3;
				break;
				
				case sixToOne:
					sound = DoExp1to6( sound, MusSize);
					MusSize *= 6;
				break;
				
				default:
				break;
			}
		break;

		case extSH:
			ExtHeader = (ExtSoundHeader*) header;
			
			MusSize = ExtHeader->numFrames;
			*sampleSize = ExtHeader->sampleSize;
			
			if( *sampleSize == 16) MusSize *= 2;
			
			HLock( sound);
			BlockMove( ExtHeader->sampleArea, *sound, MusSize);
			HUnlock( sound);
		break;
		
		case stdSH:
			MusSize = header->length;
			BlockMove( (*header).sampleArea, *sound, MusSize);
			HUnlock( sound);
		break;
	}

	ConvertInstrument( (Byte*) *sound,	MusSize);

	HUnlock( sound);
	SetHandleSize( sound, MusSize);
	
	if( MemError()) DebugStr( "\p MemError in SetHandleSize");

	return( sound);
}

/*****************************/
/****** MAIN FUNCTION ********/
/*****************************/

void main( void)
{
EventRecord		theEvent;
Point			where = { 50,50};
SFReply			reply;
short			srcFile, sampleSize;
OSErr			iErr;
long			musicSize;
DialogPtr		TheDia;
Handle			itemHandle;
Rect			itemRect;
short			itemType, i, itemHit;
Handle			mySound[ 4];			// my 4 sound resource

/* Initialisation de la toolbox */

InitGraf( &qd.thePort);
InitFonts();
InitWindows();
TEInit();
InitMenus();
InitCursor();
MaxApplZone();

/******************************************/
/*** Load and prepare my sound resource ***/
/******************************************/

for( i = 0 ; i < 4; i++)
{
	mySound[ i] = GetResource( 'snd ', 128 + i);
	if( mySound[ i] != 0L) mySound[ 0] = SndToHandle( mySound[ i], &sampleSize);		/* Convert the Snd Handle to raw handle */
	HLock( mySound[ i]);
}


/*******************************************************************************************/
/****** MAD Library Initialisation : choose the best driver for the current hardware ******/
/******																				  ******/
/****** Standard initialization with 4 tracks...									  ******/
/*******************************************************************************************/

{
MADDriverSettings	init;

init.Tracks				= 4;
init.OutPutBits 		= 8;
init.OutPutRate			= rate22khz;
init.OutPutMode			= StereoOutPut;
init.DriverMode			= SoundManagerDriver;
init.AntiAliasing		= false;
init.RepeatMusic		= false;

MADInitLibrary();
if( MADCreateDriver( &init) != noErr) Debugger();
}

MADLoadMusicRsrc( 'MADF', 3214);
MADPlay();

/******************************************/
/***        Open my dialog              ***/
/******************************************/

TheDia = GetNewDialog( 128,0L, (WindowPtr) -1L);
SetPort( TheDia);		

do
{
	ModalDialog( 0L, &itemHit);
		
	switch( itemHit)
	{
		case 2:
			MADDriver->Reading = !MADDriver->Reading;
			GetDItem( TheDia, 2, &itemType, &itemHandle, &itemRect);
			SetCtlValue( (ControlHandle) itemHandle, !GetCtlValue( (ControlHandle) itemHandle));
		break;
	
		case 3:
			MADPlaySound( *mySound[ 0], GetHandleSize( mySound[ 0]), 0, 0, sampleSize, 0, 0);		// On track ID 0, 0 = normal period
		break;
		
		case 4:
			MADPlaySound( *mySound[ 1], GetHandleSize( mySound[ 1]), 1, 0, sampleSize, 0, 0);		// On track ID 1, 0 = normal period
		break;
		
		case 5:
			MADPlaySound( *mySound[ 2], GetHandleSize( mySound[ 2]), 2, 0, sampleSize, 0, 0);		// On track ID 2, 0 = normal period
		break;
		
		case 6:
			MADPlaySound( *mySound[ 3], GetHandleSize( mySound[ 3]), 3, 0, sampleSize, 0, 0);		// On track ID 3, 0 = normal period
		break;
	}
	
}while( itemHit != 1);

DisposDialog( TheDia);

MADStop();							// Stop the driver
MADDisposeMusic();					// Clear music
MADDisposeDriver();					// Dispose driver
MADDisposeLibrary();				// Close Music Library
ExitToShell();
}

